import SocketServer
import subprocess
import os.path
import signal
import re

class MyTCPHandler(SocketServer.BaseRequestHandler):

    def handle(self):

        self.data = self.request.recv(1024).strip()  # receive the string that was sent by client and remove the extra whitepace from front and back of the string (e.g., test)

        params = self.data.split(' ')    # split the string by space ("test")

        function = params[0]    # put the first parameter in function (function = "test")

        with open('dragforce.txt','r') as f:   #open COMSOL's output file (dragforce.txt) and read it
            fd=f.read(30)
            f.close()

        with open('lastvel.txt','r') as f:   # open lastvel.txt which is generated by the interface, it gives the particle velocity from previous time step
            vel=f.read(15)
            f.close()

        if function == "test":  # if Yade's script (test.py) and Yade's output file (velserie.txt), drag force value from COMSOL (fd) and particle's velocity (vel) are available go to the function doTest
            self.doTest('test.py', 'yaderesults.txt', fd, vel)
                       
        else:
            self.request.sendall("ERROR: Function not found")  # otherwise show the message "Function not found"

        
    def doTest(self, testFile, outputFile, fd, vel):
        try:

            paramsString=fd.rstrip()   # remove the white space at the end of the string. Without this syntax, the next parameter is written on the next line.

            command = 'python yade -j8 -x '+testFile+' '+paramsString+' '+vel   # this is the command line for terminal to run Yade's script (test3.py)
            
            print 'command: '+command
            
            process = subprocess.Popen(command.strip(), shell=True)   # run the command.strip() or open the process
            
            while not os.path.isfile(outputFile):    # return true if path is an existing regular file
                pass
            
            content = ''   # content is defined as an empty string
            
            while not self.isNotEmptyContent(content):    # Open and read the outputFile if the outputFile is not empty based on the function isNotEmptyContent
                with open(outputFile, 'r') as content_file:
                    content = content_file.read()
                    content_file.close()
            
            os.kill(process.pid, signal.SIGUSR1)    # kill the process
#            os.remove(outputFile)
            
            self.request.sendall(content)
            
        except Exception as e:    # report a message "ERROR" on terminal if doTest is not completed
            self.request.sendall('ERROR: '+str(e))
            
            
    def isNotEmptyContent(self, content):
        return re.search("\w+", content)   # searching in the file for defined character in content

if __name__ == "__main__":
    HOST, PORT = "localhost", 64000
    server = SocketServer.TCPServer((HOST, PORT), MyTCPHandler)  # create the server, binding to localhost on port 64000
    server.serve_forever()   # activate the server
